#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <errno.h>
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif
#ifdef HAVE_STRING_H
# if !defined STDC_HEADERS && defined HAVE_MEMORY_H
#  include <memory.h>
# endif
# include <string.h>
#endif
#ifdef HAVE_SYS_RESOURCE_H
# include <sys/resource.h>
#endif
#ifdef HAVE_SYS_TIME_H
# include <sys/time.h>
#endif
# include <sys/types.h>
# include <sys/wait.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#ifdef HAVE_GETOPT_LONG
# include <getopt.h>
#else
# include "getopt.h"
#endif

static const char *const version=VERSION;
static const char *const progname=PACKAGE;

static int
run_cmd(int argc, char *argv[]);
static int
parse_options(int argc, char *argv[], int *nonopt_start);

static struct rusage usage;

FILE *summary_out;

int
main(int argc, char *argv[])
{
  int status, rstatus, cmd_ofs = 1;
  summary_out = stderr;
  if ((status = parse_options(argc, argv, &cmd_ofs)) != 0)
  {
    exit(status < 0?EXIT_SUCCESS:EXIT_FAILURE);
  }
/*   { */
/*     int argl = cmd_ofs; */
/*     fputs("args left:", stderr); */
/*     while(argv[argl]) */
/*       fprintf(stderr, " %s", argv[argl++]); */
/*     fputs("\n", stderr); */
/*   } */
  if ((status = run_cmd(argc - cmd_ofs, argv + cmd_ofs)) < 0)
  {
    fputs("could not fork child\n", stderr);
    return EXIT_FAILURE;
  }
  if ((rstatus = getrusage(RUSAGE_CHILDREN, &usage) == -1))
  {
    perror("resource usage statistics unavailable");
  }
  else if (rstatus != 0)
  {
    fputs("an unknown error occurred\n", stderr);
    return EXIT_FAILURE;
  }
  else
  {
    fprintf(summary_out, "%s: %ldm%3fs\n", "user time",
            usage.ru_utime.tv_sec/60,
           (usage.ru_utime.tv_sec%60 * 1000000
            + usage.ru_utime.tv_usec)/1000000.0);
    fprintf(summary_out, "%s: %ldm%3fs\n", "sys  time",
            usage.ru_stime.tv_sec/60,
           (usage.ru_stime.tv_sec%60 * 1000000
            + usage.ru_stime.tv_usec)/1000000.0);
    fprintf(summary_out, "%s: %ldk\n", "max rss", usage.ru_maxrss);
    fprintf(summary_out, "%s: %ldkiBs\n", "integral shared memory size",
            usage.ru_ixrss);
    fprintf(summary_out, "%s: %ldkiBs\n", "integral unshared data size",
            usage.ru_idrss);
    fprintf(summary_out, "%s: %ldkiBs\n", "integral unshared stack size",
            usage.ru_isrss);
    fprintf(summary_out, "%s: %ld\n", "page reclaims", usage.ru_minflt);
    fprintf(summary_out, "%s: %ld\n", "page faults", usage.ru_majflt);
    fprintf(summary_out, "%s: %ld\n", "swaps", usage.ru_nswap);
    fprintf(summary_out, "%s: %ld\n", "block input operations",
            usage.ru_inblock);
    fprintf(summary_out, "%s: %ld\n", "block output operations",
            usage.ru_oublock);
    fprintf(summary_out, "%s: %ld\n", "messages sent", usage.ru_msgsnd);
    fprintf(summary_out, "%s: %ld\n", "messages received", usage.ru_msgrcv);
    fprintf(summary_out, "%s: %ld\n", "voluntary context switches",
            usage.ru_nvcsw);
    fprintf(summary_out, "%s: %ld\n", "involuntary context switches",
            usage.ru_nivcsw);
  }
  return status;
}

static void
print_usage(FILE *out, char * const argv0);
static void
print_short_usage(char * const argv0);

static int
parse_options(int argc, char *argv[], int *nonopt_start)
{
  /* parse_args_success is returned by the function
     0 indicates successful parsing of arguments without meeting an
     exit condition.
     >0 means parsing of args has failed
     <0 means parsing of args was successful but the supplied
     options need exit of program with status 0
  */
  int parse_args_status=0;

  static const struct option long_options[]=
    {
      /*  declare valid options */
      /*  the long options are the authorative way to customize
          program operation, not all might be implemented via single letter
          versions */
      /*  optname, has arg,  */
      { "output", required_argument, 0, 'o'},
      { "help", no_argument, 0, 'h'},
      { "version", no_argument, 0, 'V'},
      { 0, 0, 0, 0}
    };

  int c;
  static const char short_options[]=":o:hV?";
  int option_index=1;

  optind = 1;
  while(!parse_args_status
        && argv[optind][0] == '-'
        && (c=getopt_long(argc, argv, short_options,
                          long_options, &option_index))!=-1)
    switch(c)
    {
    case 0:
      parse_args_status=1;
      break;
    case 'h':
    case '?':
      print_usage(stdout, argv[0]);
      parse_args_status=-1;
      break;
    case 'V':
      printf("%s %s\n", progname, version);
      parse_args_status=-1;
      break;
    case 'o':
      if(!(summary_out = fopen(optarg, "w")))
      {
        parse_args_status=1;
        fprintf(stderr, "%s: opening file \"%s\"failed: %s\n", progname,
                optarg, strerror(errno));
        break;
      }
      break;
    default:
      parse_args_status=1; /* unknown option encountered */
      print_short_usage(argv[0]);
      break;
    }
  if(parse_args_status==0)
  {
    *nonopt_start = optind;
  }
  return parse_args_status;
}

static int
run_cmd(int argc, char *argv[])
{
  pid_t pid;
  int status;
  {
    int argl = 0;
    fputs("args left:", stderr);
    while(argv[argl])
      fprintf(stderr, " %s", argv[argl++]);
    fputs("\n", stderr);
  }
  if (argc < 1)
    return -1;
  if ((pid = fork()) < 0)
    status = -1;
  else if (pid == 0)
  {
    /* child */
    execvp(argv[0], argv);
    _exit(127); /* execvp should not have returned */
  }
  else
  {
    while (waitpid(pid, &status, 0) < 0)
      if (errno != EINTR)
      {
        status = -1;
        break;
      }
  }
  return status;
}

static void
print_usage(FILE *out, char * const argv0)
{
  fprintf(
    out, "%s %s, a program for resource summary printing\n"
    "Usage: %s [OPTION...] CMD [CMDOPTS...]\n"
    "Send bug-reports to: "PACKAGE_BUGREPORT"\n"
    "Mandatory arguments to long options are mandatory for short "
    "options too.\n\n"
    "  -V,  --version      display the version of %s\n"
    "  -h,  --help         print this help.\n"
    "  -o,  --output=FILE  write to FILE instead of standard output.\n",
    progname, version, argv0, progname
    );
}

static void
print_short_usage(char * const argv0)
{
  fprintf(
    stderr, "Usage: %s [OPTION]...\n\n"
    "Try \'%s --help\' for more options.\n",
    argv0, argv0
    );
}

